{
  "type": "record",
  "Aspect": {
    "name": "origin"
  },
  "name": "Origin",
  "namespace": "com.linkedin.pegasus2avro.common",
  "fields": [
    {
      "type": {
        "type": "enum",
        "symbolDocs": {
          "EXTERNAL": "The entity is external to DataHub.",
          "NATIVE": "The entity is native to DataHub."
        },
        "name": "OriginType",
        "namespace": "com.linkedin.pegasus2avro.common",
        "symbols": [
          "NATIVE",
          "EXTERNAL"
        ],
        "doc": "Enum to define where an entity originated from."
      },
      "name": "type",
      "doc": "Where an entity originated from. Either NATIVE or EXTERNAL."
    },
    {
      "deprecated": "Use sourceDetails instead",
      "type": [
        "null",
        "string"
      ],
      "name": "externalType",
      "default": null,
      "doc": "Only populated if type is EXTERNAL. The externalType of the entity, such as the name of the identity provider."
    },
    {
      "type": [
        "null",
        {
          "type": "array",
          "items": {
            "type": "record",
            "name": "SourceDetails",
            "namespace": "com.linkedin.pegasus2avro.common",
            "fields": [
              {
                "java": {
                  "class": "com.linkedin.pegasus2avro.common.urn.Urn"
                },
                "type": "string",
                "name": "source",
                "doc": "Urn of the source of this asset. Could be an ingestion source (e.g. urn:li:dataHubIngestionSource:1234), an acryl\n    platform instance, etc.",
                "Urn": "Urn"
              },
              {
                "java": {
                  "class": "com.linkedin.pegasus2avro.common.urn.Urn"
                },
                "type": "string",
                "name": "platform",
                "doc": "Urn of the platform this asset originated from ie. urn:li:dataPlatform:snowflake for snowflake ingestion\nor urn:li:dataPlatform:acryl for assets synced from another catalog instance.",
                "Urn": "Urn"
              },
              {
                "type": {
                  "type": "record",
                  "name": "AuditStamp",
                  "namespace": "com.linkedin.pegasus2avro.common",
                  "fields": [
                    {
                      "type": "long",
                      "name": "time",
                      "doc": "When did the resource/association/sub-resource move into the specific lifecycle stage represented by this AuditEvent."
                    },
                    {
                      "java": {
                        "class": "com.linkedin.pegasus2avro.common.urn.Urn"
                      },
                      "type": "string",
                      "name": "actor",
                      "doc": "The entity (e.g. a member URN) which will be credited for moving the resource/association/sub-resource into the specific lifecycle stage. It is also the one used to authorize the change.",
                      "Urn": "Urn"
                    },
                    {
                      "java": {
                        "class": "com.linkedin.pegasus2avro.common.urn.Urn"
                      },
                      "type": [
                        "null",
                        "string"
                      ],
                      "name": "impersonator",
                      "default": null,
                      "doc": "The entity (e.g. a service URN) which performs the change on behalf of the Actor and must be authorized to act as the Actor.",
                      "Urn": "Urn"
                    },
                    {
                      "type": [
                        "null",
                        "string"
                      ],
                      "name": "message",
                      "default": null,
                      "doc": "Additional context around how DataHub was informed of the particular change. For example: was the change created by an automated process, or manually."
                    }
                  ],
                  "doc": "Data captured on a resource/association/sub-resource level giving insight into when that resource/association/sub-resource moved into a particular lifecycle stage, and who acted to move it into that specific lifecycle stage."
                },
                "name": "lastModified",
                "doc": "Audit stamp for when this asset was synced"
              },
              {
                "type": {
                  "type": "enum",
                  "symbolDocs": {
                    "API": "Synced via the API (pushed from clients using the API directly)",
                    "INGEST": "Synced via ingestion recipes",
                    "OTHER": "Synced via some other mechanism (used as a placeholder for incubating\n      future mechanisms)",
                    "SHARE": "Synced via metadata sharing with another Acryl instance"
                  },
                  "name": "SyncMechanism",
                  "namespace": "com.linkedin.pegasus2avro.common",
                  "symbols": [
                    "INGEST",
                    "SHARE",
                    "API",
                    "OTHER"
                  ]
                },
                "name": "mechanism",
                "doc": "The means by which this asset got synchronized"
              },
              {
                "type": [
                  {
                    "type": "map",
                    "values": "string"
                  },
                  "null"
                ],
                "name": "properties",
                "default": {},
                "doc": "Additional information related to the origin of this metadata"
              }
            ],
            "doc": "Info about how this asset was brought into DataHub"
          }
        }
      ],
      "name": "sourceDetails",
      "default": null,
      "doc": "Details about the source(s) by which this asset was originated"
    }
  ],
  "doc": "Carries information about where an entity originated from."
}