{
  "type": "record",
  "Aspect": {
    "name": "incidentNotificationDetails"
  },
  "name": "IncidentNotificationDetails",
  "namespace": "com.linkedin.pegasus2avro.incident",
  "fields": [
    {
      "type": [
        "null",
        {
          "type": "record",
          "name": "SlackIncidentDetails",
          "namespace": "com.linkedin.pegasus2avro.incident",
          "fields": [
            {
              "type": {
                "type": "array",
                "items": {
                  "type": "record",
                  "name": "SlackMessageDetails",
                  "namespace": "com.linkedin.pegasus2avro.incident",
                  "fields": [
                    {
                      "type": "string",
                      "name": "channelId",
                      "doc": "The original Slack channel id or user id associated with the incident message."
                    },
                    {
                      "type": [
                        "null",
                        "string"
                      ],
                      "name": "channelName",
                      "default": null,
                      "doc": "The human-readable channel name associated with the incident message.\nThis is attempted to be resolved by the integrations service.\n\nIf it cannot be found, it will be left empty."
                    },
                    {
                      "type": "string",
                      "name": "messageId",
                      "doc": "The Slack message id associated with the incident."
                    }
                  ],
                  "doc": "Slack-specific message that contains the incident information.\nNote that we will NOT capture slack message details\nfor DMs.\nThis object will be created by the integrations service once a\nnotification message about an incident is posted to a specific channel."
                }
              },
              "name": "messages",
              "default": [],
              "doc": "The Slack messages that contain the incident information"
            }
          ],
          "doc": "Slack-specific details about incident notifications"
        }
      ],
      "name": "slack",
      "default": null,
      "doc": "Details related to Slack."
    }
  ],
  "doc": "Information about the notification context for an incident (where we notified)"
}